/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 */

package com.huaweicloud.sis.android.demo.asr;

import java.io.IOException;

import java.util.Base64;

import android.app.Activity;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;

import com.huaweicloud.sdk.core.auth.BasicCredentials;
import com.huaweicloud.sdk.core.exception.ConnectionException;
import com.huaweicloud.sdk.core.exception.RequestTimeoutException;
import com.huaweicloud.sdk.core.exception.ServiceResponseException;
import com.huaweicloud.sdk.core.http.HttpConfig;
import com.huaweicloud.sdk.sis.v1.SisClient;
import com.huaweicloud.sdk.sis.v1.model.PostShortAudioReq;
import com.huaweicloud.sdk.sis.v1.model.RecognizeShortAudioRequest;
import com.huaweicloud.sdk.sis.v1.model.RecognizeShortAudioResponse;
import com.huaweicloud.sdk.sis.v1.region.SisRegion;
import com.huaweicloud.sis.android.demo.Config;
import com.huaweicloud.sis.android.demo.R;
import com.huaweicloud.sis.android.demo.service.AudioRecordService;

/**
 * 功能描述
 * 一句话识别http
 *
 * @since 2022-07-11
 */
public class SasrHttpActivity extends Activity {
    private TextView result;
    private TextView title;
    private Button startButton;
    private Handler sasrHandler;

    private HttpConfig config;
    private SisClient client;

    private AudioRecordService audioRecordService;

    @Override
    protected void onStart() {
        super.onStart();
        initView();
        initResources();
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.sasr_http);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        audioRecordService.releaseAudioRecord();
    }


    /**
     * 绑定界面数据
     */
    private void initView() {
        title = findViewById(R.id.title);
        title.setText("一句话识别(http版)");
        result = findViewById(R.id.result);
        startButton = findViewById(R.id.start);
        startButton.setText("按住说话");
        // 按钮触及事件
        startButton.setOnTouchListener(new View.OnTouchListener() {
            @RequiresApi(api = Build.VERSION_CODES.O)
            @Override
            public boolean onTouch(View view, MotionEvent event) {
                switch (event.getAction()) {
                    case MotionEvent.ACTION_UP: {
                        // 按钮松开之后触发事件
                        audioRecordService.stopAudioRecord();
                        recognitionAfterRecording();
                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                Toast.makeText(getApplicationContext(), "录音结束识别中...", Toast.LENGTH_SHORT).show();
                            }
                        });
                        break;
                    }
                    case MotionEvent.ACTION_DOWN: {
                        // 开始录音
                        audioRecordService.startRecording();
                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                Toast.makeText(getApplicationContext(), "正在进行录音中...", Toast.LENGTH_SHORT).show();
                            }
                        });
                        break;
                    }
                    default:
                        break;
                }
                return true;
            }
        });
    }

    /**
     * 初始化请求资源
     */
    private void initResources() {
        BasicCredentials auth = new BasicCredentials()
                .withAk(this.getString(R.string.HUAWEICLOUD_SDK_AK))
                .withSk(this.getString(R.string.HUAWEICLOUD_SDK_SK))
                .withProjectId(Config.PROJECT_ID);
        config = HttpConfig.getDefaultHttpConfig();
        config.withIgnoreSSLVerification(true);
        client = SisClient.newBuilder()
                .withHttpConfig(config)
                .withCredential(auth)
                .withRegion(SisRegion.valueOf(Config.REGION))
                .build();
        audioRecordService = new AudioRecordService(16000);
        sasrHandler = new Handler(getMainLooper()) {
            @Override
            public void handleMessage(@NonNull Message message) {
                super.handleMessage(message);
                switch (message.what) {
                    case 0:
                        Bundle bundle = message.getData();
                        result.setText(bundle.getString("result"));
                        break;
                    default:
                        Log.e("Unexpected value: ", String.valueOf(message.what));
                }
            }
        };
    }

    /**
     * 初始化请求参数
     */
    private RecognizeShortAudioRequest getRecognizeShortAudioRequest() {
        String encoded = null;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            encoded = Base64.getEncoder().encodeToString(audioRecordService.getByteArrayOutputStream().toByteArray());
        } else {
            encoded = android.util.Base64.encodeToString(audioRecordService.getByteArrayOutputStream().toByteArray(), 2);
        }
        audioRecordService.closeByteArrayOutStream();
        com.huaweicloud.sdk.sis.v1.model.Config configbody = new com.huaweicloud.sdk.sis.v1.model.Config();
        configbody.setAudioFormat(com.huaweicloud.sdk.sis.v1.model.Config.AudioFormatEnum.fromValue("pcm16k16bit"));
        configbody.setProperty(com.huaweicloud.sdk.sis.v1.model.Config.PropertyEnum.fromValue("chinese_16k_general"));
        configbody.setAddPunc(com.huaweicloud.sdk.sis.v1.model.Config.AddPuncEnum.YES);
        RecognizeShortAudioRequest request = new RecognizeShortAudioRequest();
        PostShortAudioReq body = new PostShortAudioReq();
        body.withData(encoded);
        body.withConfig(configbody);
        request.withBody(body);
        return request;
    }


    /**
     * 录音完成开始识别
     *
     * @throws IOException 输入异常
     */
    private void recognitionAfterRecording() {
        new Thread(() -> {
            RecognizeShortAudioRequest request = getRecognizeShortAudioRequest();
            String sasrRequestText = getSasrResponse(request);
            Message message = new Message();
            Bundle mBundle = new Bundle();
            mBundle.putString("result", sasrRequestText);
            message.setData(mBundle);
            sasrHandler.sendMessage(message);
        }).start();
    }


    /**
     * 一句话识别请求。
     *
     * @return 返回识别的文字
     */
    private String getSasrResponse(RecognizeShortAudioRequest request) {
        String resultStr = "";
        try {
            RecognizeShortAudioResponse response = client.recognizeShortAudio(request);
            resultStr = response.getResult().getText();
            Log.i("info:", response.getResult().getText());
        } catch (ConnectionException | RequestTimeoutException | ServiceResponseException e) {
            resultStr = e.toString();
            Log.i("error:", e.toString());
        }
        return resultStr;
    }
}
