/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 */

package com.huaweicloud.sis.android.demo.asr;

import java.util.concurrent.atomic.AtomicBoolean;

import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

import com.huaweicloud.sdk.core.utils.JsonUtils;
import com.huaweicloud.sis.android.demo.R;
import com.huaweicloud.sis.android.demo.service.AudioRecordService;
import com.huaweicloud.sis.android.demo.Config;

import sis.android.sdk.SasrWsClient;
import sis.android.sdk.bean.AuthInfo;
import sis.android.sdk.bean.SisHttpConfig;
import sis.android.sdk.bean.request.SasrWsRequest;
import sis.android.sdk.bean.response.AsrResponse;
import sis.android.sdk.exception.SisException;
import sis.android.sdk.listeners.SasrWsResponseListener;
import sis.android.sdk.listeners.process.SasrWsConnProcessListener;

/**
 * 功能描述
 * 一句话识别websocket
 *
 * @since 2022-07-11
 */
public class SasrWsActivity extends AppCompatActivity {
    private TextView result;
    private Button startButton;
    private AudioRecordService audioRecordService;
    private AuthInfo authInfo;
    private SasrWsClient sasrWsClient;

    // 实时显示识别的结果
    private StringBuffer realTimeResult;

    // 是否需要发送end请求
    private AtomicBoolean sendEndFlag = new AtomicBoolean(false);

    private SasrWsResponseListener sasrWsResponseListener = new SasrWsResponseListener() {
        @Override
        public void onExceededAudio() {
            sendEndFlag.set(true);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Toast.makeText(getApplicationContext(), "超过1分钟，识别结束...", Toast.LENGTH_SHORT).show();
                }
            });
        }

        // 一句话识别 回调结果更新到界面UI中
        @Override
        public void onResponseMessage(AsrResponse asrResponse) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    for (int i = 0; i < asrResponse.getSegments().size(); i++) {
                        AsrResponse.Segment segment = asrResponse.getSegments().get(i);
                        result.setText(realTimeResult.toString() + segment.getResult().getText());
                        if (segment.getIsFinal()) {
                            realTimeResult.append(segment.getResult().getText());
                        }
                    }
                }
            });
        }

        /**
         * 开始时响应
         *
         * @param response 返回体
         */
        @Override
        public void onResponseBegin(AsrResponse response) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Toast.makeText(getApplicationContext(), "连接成功，请录入音频...", Toast.LENGTH_SHORT).show();
                }
            });
        }

        @Override
        public void onResponseEnd(AsrResponse response) {
        }

        /**
         * 传输过程中的错误时响应
         *
         * @param response 返回体
         */
        @Override
        public void onResponseError(AsrResponse response) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    // 调用失败给用户提示
                    Toast.makeText(getApplicationContext(), JsonUtils.toJSON(response), Toast.LENGTH_SHORT).show();
                }
            });
        }
    };

    private SasrWsConnProcessListener sasrWsConnProcessListener = new SasrWsConnProcessListener() {
        /**
         * 连接建立后回调
         */
        @Override
        public void onTranscriptionConnect() {
            Log.i("info", "长连接开始");
        }

        /**
         * 连接关闭后回调
         */
        @Override
        public void onTranscriptionClose() {
            Log.i("info", "长连接关闭");
        }

        /**
         * 长连接连接失败是回调
         *
         * @param asrResponse 返回体
         */
        @Override
        public void onTranscriptionFail(AsrResponse asrResponse) {
            Log.i("info", "长连接异常,等待中");
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    // 调用失败给用户提示
                    Toast.makeText(getApplicationContext(), JsonUtils.toJSON(asrResponse), Toast.LENGTH_SHORT).show();
                }
            });
        }
    };


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.sasr_websocket);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (audioRecordService != null) {
            audioRecordService.releaseAudioRecord();
        }
    }

    @Override
    protected void onStart() {
        super.onStart();
        initView();
        initResources();
    }

    /**
     * 初始化UI界面
     */
    private void initView() {
        result = findViewById(R.id.result);
        startButton = findViewById(R.id.start);
        startButton.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View view, MotionEvent event) {
                switch (event.getAction()) {
                    case MotionEvent.ACTION_UP:
                        audioRecordService.stopAudioRecord();
                        if (!sendEndFlag.get()) {
                            try {
                                sasrWsClient.sendEnd();
                            } catch (SisException e) {
                                Log.e("error", e.getErrorCode() + e.getErrorMsg());
                            }
                            sendEndFlag.set(false);
                        }
                        sasrWsClient.close();
                        break;
                    case MotionEvent.ACTION_DOWN:
                        try {
                            actionDown();
                        } catch (SisException e) {
                            Log.e("error", e.getErrorCode() + e.getErrorMsg());
                        }
                        break;
                    default:
                        break;
                }
                return true;
            }
        });
    }

    /**
     * 初始化请求资源
     */
    private void initResources() {
        authInfo = new AuthInfo(this.getString(R.string.HUAWEICLOUD_SDK_AK), this.getString(R.string.HUAWEICLOUD_SDK_SK),
                Config.REGION, Config.PROJECT_ID);
        audioRecordService = new AudioRecordService(16000);
        realTimeResult = new StringBuffer();
    }

    /**
     * 开始请求
     */
    private SasrWsRequest getStartRequest() {
        SasrWsRequest sasrWsRequest = new SasrWsRequest();
        sasrWsRequest.setCommand("START");
        SasrWsRequest.Config config = new SasrWsRequest.Config();
        config.setAudioFormat("pcm16k16bit");
        config.setProperty("chinese_16k_common");
        config.setAddPunc("yes");
        config.setInterimResults("yes");
        sasrWsRequest.setConfig(config);
        return sasrWsRequest;
    }

    /**
     * 按钮按下之后事件
     *
     * @throws SisException
     */
    private void actionDown() throws SisException {
        realTimeResult = realTimeResult.delete(0, realTimeResult.length());
        try {
            sasrWsClient = new SasrWsClient(authInfo, sasrWsResponseListener, sasrWsConnProcessListener, new SisHttpConfig());
            audioRecordService.getAudioRecord().startRecording();
            sasrWsClient.connect();
            sasrWsClient.sendStart(getStartRequest());
            audioRecordService.startSendRecordingData(sasrWsClient);
        } catch (SisException e) {
            Log.e("error", e.getErrorCode() + e.getErrorMsg());
        }
    }
}
